/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractListModel;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Eine Liste von Datenspezifikationen. Gleichzeitig das Model fr die JListe in der GUI
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.1 $ / $Date: 2008/01/22 16:39:42 $ / ($Author: yvonnes $)
 */
public class DataspecificationList extends AbstractListModel {
    /**
     * 
     */
    private static final long serialVersionUID = 7756247055009613023L;
    List<Dataspecification> Dataspec;


    /**
     * Erzeugt eine neue DataspecificationList-Instanz, die keine Daten enthlt
     */
    public DataspecificationList() {
        Dataspec = new ArrayList<Dataspecification>();
    }


    /**
     * Erzeugt eine neue DataspecificationList-Instanz, die die bergebenen Datenspezifikationen
     * enthlt
     * 
     * @param Dataspec Eine Liste an Datenspezifikationen, mit der die DataspecificationList
     * initialisiert wird
     */
    public DataspecificationList(List<Dataspecification> Dataspec) {
        this.Dataspec = Dataspec;
    }


    /**
     * Fgt der DataspecificationList ein Datenspezifikations-Objekt at
     * 
     * @param dataspecification Dataspecification, die hinzugefgt werden soll
     */
    public void add(Dataspecification dataspecification) {
        Dataspec.add(dataspecification);
        fireIntervalAdded(this, Dataspec.size() - 1, Dataspec.size());
    }


    /**
     * Fgt der DataspecificationList ein Datenspezifikations-Objekt at
     * 
     * @param id
     * 
     * @param dataspecification Dataspecification, die hinzugefgt werden soll
     */
    public void update(int id, Dataspecification dataspecification) {
        assert id >= 0 : "Negative ID";
        assert id < Dataspec.size() : "ID grer als IDmax";

        Dataspec.remove(id);
        Dataspec.add(id, dataspecification);
        fireContentsChanged(this, id, id);
    }


    /**
     * Ersetzt die Liste der Datenspezifikationen vollstndig mit der bergebenen.
     * 
     * @param Dataspecifications die neue Liste an Datenspezifikationen
     */
    public void replaceDataspecificationWith(List<Dataspecification> Dataspecifications) {
        Dataspec = Dataspecifications;
        fireContentsChanged(this, 0, Dataspec.size());
    }


    /**
     * Entfernt die mittels Index angegebenen Dataspecification TODO: Reagieren, wenns das Element
     * nicht gibt
     * 
     * @param index Index der Dataspecification, die entfert werden soll
     */
    public void remove(int index) {
        if (Dataspec.size() >= index) {
            Dataspec.remove(index);
        } else {
            System.out.println("KEIN ELEMTENT bei Index " + index + "! Size: " + Dataspec.size());
        }
        fireIntervalRemoved(this, index, index);
    }


    /**
     * Element mit Index index zurckliefern, oder null, wenn es an unter diesem index kein Element
     * gibt
     * 
     * @param index Der Index des angeforderten Elementes
     * @return Das Element mit Index index
     */
    public Object getElementAt(int index) {
        if (Dataspec.size() >= index) {
            return Dataspec.get(index);
        } else {
            return null;
        }
    }


    /**
     * Gre der Liste
     * 
     * @return Gre der Liste
     */
    public int getSize() {
        return Dataspec.size();
    }


    /**
     * @param file
     * @throws IOException
     */
    public void saveToFile(File file) throws IOException {
        FileWriter fw = new FileWriter(file);
        fw.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");

        fw.write("<datenspezifikationen created=\"" + ExportProfile.dateFormat.format(new Date()) + "\">\n");

        for (Iterator i = Dataspec.iterator(); i.hasNext();) {
            Dataspecification ds = (Dataspecification) i.next();
            fw.write("\t<datenspezifikation" + generateAttribute("name", ds.getName()));
            fw.write(generateAttribute("asp", ds.getAsp().getId() + ""));
            fw.write(generateAttribute("atg", ds.getAtg().getId() + ""));
            fw.write(generateAttribute("simvar", ds.getSimVar() + ""));
            fw.write(">\n");

            for (Iterator j = ds.getSystemObjects().iterator(); j.hasNext();) {
                SystemObject so = (SystemObject) j.next();
                fw.write("\t\t<objekt" + generateAttribute("id", Long.toString(so.getId())) + " />\n");
            }

            fw.write("\t</datenspezifikation>\n");
        }

        fw.write("</datenspezifikationen>\n");
        fw.close();
    }


    /**
     * Nimmt einen String und ersetzt darin alle kritischen XML-Zeichen durch deren EscapeSeuqenzen.
     * 
     * @param s String, auf dem die Ersetzung stattfinden soll
     * @return bereinigter String
     */
    private String escapeString(String s) {
        StringBuffer b = new StringBuffer((int) (s.length() * 0.1));
        char c;

        for (int i = 0; i < s.length(); i++) {
            c = s.charAt(i);
            switch (c) {
                case '<':
                    b.append("&lt;");
                    break;
                case '>':
                    b.append("&gt;");
                    break;
                case '\"':
                    b.append("&quot;");
                    break;
                case '&':
                    b.append("&amp;");
                    break;
                case '\r':
                    b.append("&#xD;");
                    break;
                case '\t':
                    b.append("&#x9;");
                    break;
                case '\n':
                    b.append("&#xA;");
                    break;
                default:
                    b.append(c);
            }
        }
        return b.toString();
    }


    /**
     * Hilfsfunktion: Erzeugt ein Attribut in der Form name="wert" Der Wert wird escaped, vor name
     * ist ein Leerzeichen
     * 
     * @param name Name des Attributes
     * @param value Wert des Attributes
     * @return Name-Wert-Paar
     */
    private String generateAttribute(String name, String value) {
        return " " + name + "=\"" + escapeString(value) + "\"";
    }
}
